@@ -11,7 +11,7 @@ import itertools
 import math  # lgtm [py/unused-import]
 import textwrap
 import types
-from typing import Any, List, Union
+from typing import Any, Iterable, List, Union
 
 import sympy
 from sympy.parsing.sympy_parser import (
@@ -56,7 +56,7 @@ def collect_until_false(
         val = safe_apply(function, val, ctx=ctx)
 
 
-def concat(vec1: VyList, vec2: VyList, ctx: Context = None) -> VyList:
+def concat(vec1: VyList, vec2: VyList, ctx: Context = DEFAULT_CTX) -> VyList:
     """Concatenate two lists/lazylists"""
     if LazyList not in (type(vec1), type(vec2)):
         return vec1 + vec2
@@ -218,6 +218,14 @@ def get_input(ctx: Context) -> Any:
                 return 0
 
 
+def has_ind(lst: VyList, ind: int) -> bool:
+    """Whether or not the list is long enough for that index"""
+    if isinstance(lst, LazyList):
+        return lst.has_ind(ind)
+    else:
+        return 0 <= ind < len(lst)
+
+
 def indent_str(string: str, indent: int, end="\n") -> str:
     """Indent a multiline string with 4 spaces, with a newline or `end` afterwards."""
     return textwrap.indent(string, "    " * indent) + end
@@ -246,6 +254,11 @@ def invert_brackets(lhs: str) -> str:
     return res
 
 
+def is_inf(lst: VyList) -> bool:
+    """Whether or not a list/LazyList is infinite"""
+    return isinstance(lst, LazyList) and lst.infinite
+
+
 def is_sympy(value):
     """Whether or not this is a Sympy type"""
     return isinstance(value, sympy.Basic)
@@ -280,7 +293,7 @@ def levenshtein_distance(s1: str, s2: str) -> int:
     if len(s1) > len(s2):
         s1, s2 = s2, s1
 
-    distances = range(len(s1) + 1)
+    distances: Iterable[int] = range(len(s1) + 1)
     for i2, c2 in enumerate(s2):
         distances_ = [i2 + 1]
         for i1, c1 in enumerate(s1):
@@ -316,14 +329,10 @@ def local_maxima(lhs: str) -> List[Union[int, float]]:
 
 def keep(haystack: Any, needle: Any) -> Any:
     """Used for keeping only needle in haystack"""
-    out = "" if type(haystack) is str else []
-    for item in haystack:
-        if item in needle:
-            if type(haystack) is str:
-                out += item
-            else:
-                out.append(item)
-    return out
+    if isinstance(haystack, str):
+        return "".join(char for char in haystack if char in needle)
+    else:
+        return LazyList(item for item in haystack if item in needle)
 
 
 def make_equation(eqn: str) -> sympy:
@@ -537,6 +546,23 @@ def pop(iterable_object: VyList, count: int, ctx: Context) -> List[Any]:
     return popped_items
 
 
+def prefixes(lhs: Union[VyList, str], ctx: Context) -> VyList:
+    """Returns a list of prefixes of a string or list
+    (not including [] or '')"""
+    if isinstance(lhs, str):
+        return [lhs[: i + 1] for i in range(len(lhs))]
+    else:
+
+        @lazylist
+        def gen():
+            temp = []
+            for item in iterable(lhs, ctx=ctx):
+                temp.append(deep_copy(item))
+                yield temp
+
+        return gen()
+
+
 def primitive_type(item: Any) -> Union[str, type]:
     """Turns int/Rational/str into 'Scalar' and everything else
     into list"""
@@ -588,8 +614,7 @@ def safe_apply(function: types.FunctionType, *args, ctx) -> Any:
         else:
             return []
     elif function.__name__.startswith("VAR_"):
-        ret = function(list(args)[::-1], function, ctx=ctx)[-1]
-        return ret
+        return function(list(args)[::-1], function, ctx=ctx)[-1]
     elif takes_ctx(function):
         return function(*args, ctx=ctx)
     return function(*args)
@@ -646,6 +671,8 @@ def simplify(value: Any) -> Union[int, float, str, list]:
         return value
     elif is_sympy(value):
         return eval(sympy.pycode(value))
+    elif isinstance(value, types.FunctionType):
+        return str(value)
     else:
         return [simplify(x) for x in value]
 
@@ -659,9 +686,21 @@ def stationary_points(lhs: str) -> List[Union[int, float]]:
     return LazyList(zeros)
 
 
-def suffixes(string: str, ctx: Context) -> List[str]:
-    """Returns a list of suffixes of string"""
-    return [string[-i:] for i in range(len(string))]
+def suffixes(lhs: Union[str, VyList], ctx: Context) -> VyList:
+    """Returns a list of suffixes, including the original list"""
+    if isinstance(lhs, str):
+        return [lhs[-i:] for i in range(len(lhs))]
+
+    lst = iterable(lhs, ctx=ctx)
+
+    @lazylist
+    def gen():
+        nonlocal lst
+        while lst:
+            yield lst
+            lst = lst[1:]
+
+    return gen()
 
 
 def takes_ctx(function: types.FunctionType) -> bool:
@@ -675,7 +714,7 @@ def to_base_digits(value: int, base: int) -> List[int]:
     ret = []
     n = value
 
-    while n > base:
+    while n >= base:
         n, digit = divmod(n, base)
         ret.append(digit)
     ret.append(n)
@@ -706,7 +745,7 @@ def transfer_capitalisation(source: str, target: str) -> str:
 
 
 def transpose(
-    vector: VyList, filler: Any = None, ctx: Context = None
+    vector: VyList, filler: Any = None, ctx: Context = DEFAULT_CTX
 ) -> VyList:
     """Transposes a vector"""
     vector = iterable(vector, ctx=ctx)
@@ -840,9 +879,9 @@ def vyxalify(value: Any) -> Any:
         return LazyList(map(vyxalify, value))
 
 
-def wrap_with_width(vector: Union[str, list], width: int) -> List[Any]:
+def wrap_with_width(vector: Union[str, list], width: int) -> list[Any]:
     """A version of textwrap.wrap that plays nice with spaces"""
-    ret = []
+    ret: list[Union[str, list]] = []
     temp = []
     for item in vector:
         temp.append(item)
@@ -861,7 +900,9 @@ def wrap_with_width(vector: Union[str, list], width: int) -> List[Any]:
     return ret
 
 
-def wrapify(item: Any, count: int = None, ctx: Context = None) -> List[Any]:
+def wrapify(
+    item: Any, count: int = None, ctx: Context = DEFAULT_CTX
+) -> List[Any]:
     """Leaves lists as lists, wraps scalars into a list"""
     if count is not None:
         temp = pop(item, count, ctx)
